#
# Rules for using Scheme while making the TUNES project
# Rules.make,v 1.1.1.1 1998/11/26 04:10:48 tunes Exp
#

################################ CONFIGURATION ###############################

# When using the Tunes Scheme library without TUNES,
# the following make variables must be defined:

# SCHEME_LIB_PATH should be defined,
# and point to a directory containing the Tunes Scheme library
# it can be sort of autodetected,
# if it's a (symlink) subdirectory "lib" of the current directory,
# or if $TUNES is properly defined.

# SCHEME_TYPE should be defined and point to the type of Scheme interpreter,
# such that init/$(SCHEME_TYPE).scm defines the proper TUNES compatibility
# layer required for the library to support the interpreter

# SCHEME should be defined and point to a Scheme interpreter.
# it can be sort of autodetected if SCHEME_TYPE is defined

# Down with lack of standardization of even survival features in RnRS!
# At least there should be a library standardization committee, so that
# semantically equivalent (optional) features have the same name when
# available, and same-named features have same semantics! And a minimal
# standard module system would be darn useful, too!

##############################################################################

ifndef SCHEME_LIB_PATH
SCHEME_LIB_PATH = $(shell \
	for i in $$SCHEME_LIB_PATH $(TUNES)/src/HLL/scm `pwd`/lib ; do \
	   if [ -f $$i/lib.scm ] ; then echo $$i ; exit 0 ; fi \
	done ; echo "Couldn't find TUNES Scheme library!" ; exit 1)
endif

ifndef SCHEME
echo ERROR: no Scheme interpreter defined.
endif

ifndef SCHEME_TYPE
echo ERROR: Scheme interpreter's type not given.
endif

ifeq (,$(wildcard $(SCHEME_LIB_PATH)/init/$(SCHEME_TYPE).scm))
echo ERROR: Scheme interpreter's type not supported.
endif

### $(SCHEME) and $(SCHEME_TYPE) must be consistently defined.
# Search for CONFIG_Scheme_ in $(TUNES)/bin/autoconf.sh for consistent values.
# Examples (with =foo to be expanded into full path)
# =snow stk
# =stk stk
# =elk elk
# =mzscheme mzscheme
# =rs rs
# =gsi gambit
# =sci scc
#
### Scheme Compiler-only support not ready yet.
### $(SCHEME_COMP) $(SCHEME_TYPE) $(SCHEME_COMP_FLAGS)
###	$(SCHEME_LINK)
###	$(SCHEME_COMP_HEADERS)
# =rsc rs ???
# =gsc gambit "-verbose -report -o compiled.c"
# 	"gcc -O2 -Wall -o compiled compiled.c -lgambc"
# =scc scc ???
# =bigloo bigloo "-O2 -farithmetic -fsharing -q -Wall -o compiled"
#	""
#	"(module $%)"

##############################################################################

# defines target suffixes .scm-init .scm-compiled
# defines dummy target suffixes .scm-test .scm-comptest
.SUFFIXES:	.scm .sc \
		.scm-init .scm-compiled \
		.scm-test .scm-comptest
.PHONY:		phonyphony

phonyphony:

##############################################################################

#### Autodefinition of SCHEME_FLAGS

ifeq ($(SCHEME_TYPE),guile)
ifndef SCHEME
SCHEME = guile
endif
SCHEME_FLAGS = -s
endif

ifeq ($(SCHEME_TYPE),elk)
ifndef SCHEME
SCHEME = elk
endif
SCHEME_FLAGS = -l
endif

ifeq ($(SCHEME_TYPE),stk)
ifndef SCHEME
SCHEME = snow
endif
SCHEME_FLAGS = -f
endif

ifeq ($(SCHEME_TYPE),mzscheme)
ifndef SCHEME
SCHEME = mzscheme
endif
SCHEME_FLAGS = -f
endif

ifeq ($(SCHEME_TYPE),rs)
ifndef SCHEME
SCHEME = rs
endif
SCHEME_FLAGS = -exit
endif

##############################################################################

%.scm-init: %.scm phonyphony
	@( \
	echo "(define *tunes-lib-path* \"$(SCHEME_LIB_PATH)\")";\
	echo "(define *tunes-tmp-prefix* \"/tmp/tunestmp-$$HOSTNAME-$$$$\")";\
	echo "(load \"$(SCHEME_LIB_PATH)/init/foo.scm\")";\
	echo "(load \"$(SCHEME_LIB_PATH)/init/$(SCHEME_TYPE).scm\")";\
	echo "(load \"$<\")";\
	) > $@

%.scm-test: %.scm-init
	$(SCHEME) $(SCHEME_FLAGS) $<

# 2>&1 | \
# perl -npe ' \
#	s/\(\(magic\) ([a-z]*)\)/\#\1/g ; \
#'

%.scm-compiled: %.scm-init
	@(echo $(SCHEME_HEADER) ;\
	echo "(define (provide x) #t)" ;\
	$(TUNES)/bin/depend.pl merge $< ) > compiled.scm
%.compiled: %.scm-compiled
	$(SCHEME_COMP) %.scm-compiled
	$(SCHEME_LINK)

%.comptest: %.compiled
	./compiled
