;;;;
;;;; s y n t e n v . s c m	     -- syntactic environments
;;;;					(lists of bindings)
;;;;

;(#include "stdheader.h")

(provide 'syntenv)
(require 'lib)
(require 'bintree)


"/;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\"
Syntactic environments:

A binding is an association pair indicating
that a symbol is bound to some value.
cons cells are not used, for the reasons listed in the 'vectree module,
as well as because this allows the same infrastructure
to be later used for proof environments,
where instead of symbol->value binding
(understood as the axiom ``this symbol, as an expression,
reduces to this value''), we can have arbitrary axioms.

A syntactic environment is a list of bindings,

	

   we have 
* a target environment accepts new symbol -> object bindings.
"/;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\"

bindings, as used in
syntactic environments

;;;
;;; Basic constructors and destruct(urat)ors
;;;
(define binding-tag (make-tag 'BIND))
(define (make-binding sym val) (vector binding-tag sym val))
(define (binding? x) (vectree binding-tag 2 x))
(define binding:sym  v-1st)
(define binding:val  v-2nd)
(define binding:set-sym!  v-set-1st!)
(define binding:set-val!  v-set-2nd!)

; Syntactic Environment
; implemented as a particular case of association tables,
; where the key is a symbol
; environments are done with binary trees so that appending
; environments be cheap without compromising lookup cost.


; Syntactic environment
(define syntenv:nil bt:nil)
(define syntenv:new bt:new)
(define (syntenv:bind-cons sym val env) (bt:cons)
(define syntenv:find assoc:lookup)
(define syntenv:find assoc:find)
(define syntenv:join assoc:merge)

(define syntenv:cons bt:cons)


(define assoc:nil '())
(define (assoc:new) assoc:nil)
(define (assoc:cons key value table) (cons (cons key value) table))
(define (assoc:find table fun found not-found)
  (lookup->find list:lookup table fun found not-found))
(define (assoc:findq table key found not-found)
	(assoc:lookup table (compareq-to key) found not-found))
(define (assoc:findv table key found not-found)
	(assoc:lookup table (comparev-to key) found not-found))
(define assoc:merge append)


; Stuff associated to symbols:
; VALUE (quote-pattern)
; PATTERN
; SPECIAL-PATTERN -> constructing and deconstructing functions
;
;might be implemented as dynamically appended list of lists
;(define (syntenv:cons x v env)
;    (cons (assoc:cons x v (car env)) (cdr env)))
